__author__ = 'Helium'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


#parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of description page
#return: 'row' that contains a variety of lists that each hold info on the description page
def lionmarketplace_description_parser(soup):

    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo
    image = "-1"                        # 19 Product_Image
    vendor_image = "-1"                 # 20 Vendor_Image

    # vendor name
    temp = soup.find('div', {'class': 'btn-group'}).find('a').text
    vendor = (cleanString(temp.strip()))

    # table with info
    table = soup.find('table')
    rows = table.findAll('tr')

    # successful transaction
    success = "-1"

    # vendor rating 5
    rating_vendor = '-1'

    # product name
    temp = soup.find('div', {'class', 'row'}).find('h2').text
    name = (cleanString(temp.strip()))

    # product description
    temp = soup.find('div', {'class': "mt-4"}).contents[-1]
    describe = cleanString(temp.strip())

    # Finding Product Image
    image = soup.find('div', {'id': 'slide-1'}).find('img')
    image = image.get('src')
    image = image.split('base64,')[-1]

    CVE = "-1"  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures) dont worry about that much
    MS = "-1"  # 6 Product_MS_Classification (Microsoft Security) dont worry about that much

    # product category
    temp = rows[1].find('strong').text
    category = cleanString(temp.strip())

    # product number of views
    views = "-1"
    reviews = "-1"  # 9 Product_Number_Of_Reviews
    rating_item = "-1"  # 10 Product_Rating
    addDate = "-1"  # 11 Product_AddedDate
    # BTC selling price box box-rounded mt-2
    BTC = "-1"

    # USD selling price
    temp = rows[2].find('strong').text
    if " $" in temp:
        temp = temp.replace(" $", "")
    elif "$" in temp:
        temp = temp.replace("$", "")
    USD = cleanString((temp.strip()))

    EURO = "-1"  # 14 Product_EURO_SellingPrice

    # product sold
    if (len(rows) <= 5):
        temp = rows[4].find('td').text
        string = cleanString(temp)
        if (string == 'Left/Sold'):
            temp = rows[4].findAll('td')
            temp = temp[1].findAll('span')

            # left
            temp2 = temp[1].text
            temp3 = temp[1].text

            if(" items" in temp2):
                temp2 = temp2.replace(" items", "")
            if(" items" in temp3):
                temp3 = temp3.replace(" items", "")

            sold = (cleanString(temp2.strip()))
            left = cleanString(temp3.strip())
        else:
            sold = '-1'
            left = "-1"
    else:
        sold = '-1'
        left = "-1"

    shipFrom = "-1"  # 17 Product_ShippedFrom
    shipTo = "-1"  # 18 Product_ShippedTo

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)

    # Sending the results
    return row


#parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of listing page
#return: 'row' that contains a variety of lists that each hold info on the listing page
def lionmarketplace_listing_parser(soup):

    # Fields to be parsed
    nm = 0                                    # *Total_Products (Should be Integer)
    mktName = "LionMarketplace"               # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security)
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft =[]                                 # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    image = []                                # 20 Product_Image
    image_vendor = []                         # 21 Vendor_Image
    href = []                                 # 22 Product_Links

    listings = soup.findAll('div', {"class": "col-md-4 my-md-0 my-2 col-12"})

    # Populating the Number of Products
    nm = len(listings)

    for listing in listings:

        a = listing.find('div', {"class": "card-body"})
        row = a.findAll('p')

        # vendor
        temp = row[3].text
        temp = temp.replace("Vendor:", "")
        vendor.append(cleanString(temp.strip()))

        image_vendor.append("-1")

        # vendor rating
        rating_vendor.append("-1")

        # successful transactions CHECK AGAIN HERE
        success.append("-1")

        # product name
        temp = a.find('a').text
        name.append(cleanString(temp.strip()))

        # Finding Product Image
        product_image = listing.find('img', {'class': 'card-img-top rounded'})
        product_image = product_image.get('src')
        product_image = product_image.split('base64,')[-1]
        image.append(product_image)

        CVE.append('-1')
        MS.append('-1')

        # product category
        temp = row[2].text
        temp = temp.replace("Category: ", "")
        category.append(cleanString(temp.strip()))

        describe.append('-1')

        # product views
        views.append("-1")
        reviews.append('-1')  # 10 Product_Number_Of_Reviews
        rating_item.append('-1')  # 11 Product_Rating
        addDate.append('-1')  # 12 Product_AddDate

        # BTC
        BTC.append('-1')

        # USD
        temp = row[0].find('strong').text
        if ' $' in temp:
            temp = temp.replace(" $", "")
        USD.append(cleanString(temp.strip()))  # 14 Product_USD_SellingPrice

        EURO.append("-1")  # 15 Product_EURO_SellingPrice

        # product sold
        sold.append("-1")

        qLeft.append('-1')  # 17 Product_QuantityLeft
        shipFrom.append('-1')  # 18 Product_ShippedFrom
        shipTo.append('-1')  # 19 Product_ShippedTo

        # href
        temp = a.find('a').get('href')
        href.append(temp)

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href, image, image_vendor)


#called by the crawler to get description links on a listing page
#@param: beautifulsoup object that is using the correct html page (listing page)
#return: list of description links from a listing page
def lionmarketplace_links_parser(soup):

    # Returning all links that should be visited by the Crawler

    href = []
    listings = soup.findAll('div', {"class": "col-md-4 my-md-0 my-2 col-12"})

    for listing in listings:
        a = listing.find('div', {"class": "card-body"})
        bae = a.find('a', href=True)
        link = bae['href']
        href.append(link)

    return href