__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup

import re

#parses description pages, so takes html pages of description pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of description page
#return: 'row' that contains a variety of lists that each hold info on the description page
def nexus_description_parser(soup):

    # Fields to be parsed

    vendor = "-1"                       # 0 *Vendor_Name
    success = "-1"                      # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"                # 2 Vendor_Rating
    name = "-1"                         # 3 *Product_Name
    describe = "-1"                     # 4 Product_Description
    CVE = "-1"                          # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                           # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                     # 7 Product_Category
    views = "-1"                        # 8 Product_Number_Of_Views
    reviews = "-1"                      # 9 Product_Number_Of_Reviews
    rating_item = "-1"                  # 10 Product_Rating
    addDate = "-1"                      # 11 Product_AddedDate
    BTC = "-1"                          # 12 Product_BTC_SellingPrice
    USD = "-1"                          # 13 Product_USD_SellingPrice
    EURO = "-1"                         # 14 Product_EURO_SellingPrice
    sold = "-1"                         # 15 Product_QuantitySold
    left = "-1"                         # 16 Product_QuantityLeft
    shipFrom = "-1"                     # 17 Product_ShippedFrom
    shipTo = "-1"                       # 18 Product_ShippedTo
    image = "-1"                        # 19 Product_Image
    vendor_image = "-1"                 # 20 Vendor_Image


    #finding the name of the product
    name_of_product = soup.find("h1", {"class": "product_title entry-title"}).text
    name = cleanString(name_of_product.strip())

    # Find the BTC Price
    prices = soup.find('p', {"class": "price"}).findAll('span', {"class": "cs"})
    BTC = prices[0].text
    BTC = cleanNumbers(BTC.strip())

    # finding the description of the product
    description_div = soup.find("div", {"class": "woocommerce-product-details__short-description"})
    if description_div is None:
        describe = "-1"
    else:
        describe = cleanString(description_div.text.strip())

    # Finding Product Image
    image = soup.find('div', {'class': 'woocommerce-product-gallery__wrapper'}).find('img')
    image = image.get('src')
    image = image.split('base64,')[-1]

    #find the category of the product
    name_of_category = soup.find("span", {"class": "posted_in"}).find("a").text
    category = cleanString(name_of_category.strip())

    #finding the name of the vendor
    name_of_vendor = soup.find("div", {"class": "dokan-vendor-name"}).find("h5").text
    vendor = cleanString(name_of_vendor)

    #finding the vendor's rating
    vendorRating = soup.find("div", {"class": "dokan-vendor-rating"}).find("p").text
    rating_vendor = cleanString(vendorRating)
    #everything else gets a -1 because they are not found

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo, image, vendor_image)


    # Sending the results
    return row


#parses listing pages, so takes html pages of listing pages using soup object, and parses it for info it needs
#stores info it needs in different lists, these lists are returned after being organized
#@param: soup object looking at html page of listing page
#return: 'row' that contains a variety of lists that each hold info on the listing page
def nexus_listing_parser(soup):

    # Fields to be parsed
    nm = 0                                    # *Total_Products (Should be Integer)
    mktName = "Nexus"                         # 0 *Marketplace_Name
    vendor = []                               # 1 *Vendor y
    rating_vendor = []                        # 2 Vendor_Rating
    success = []                              # 3 Vendor_Successful_Transactions
    name = []                                 # 4 *Product_Name y
    CVE = []                                  # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []                                   # 6 Product_MS_Classification (Microsoft Security)
    category = []                             # 7 Product_Category y
    describe = []                             # 8 Product_Description
    views = []                                # 9 Product_Number_Of_Views
    reviews = []                              # 10 Product_Number_Of_Reviews
    rating_item = []                          # 11 Product_Rating
    addDate = []                              # 12 Product_AddDate
    BTC = []                                  # 13 Product_BTC_SellingPrice
    USD = []                                  # 14 Product_USD_SellingPrice y
    EURO = []                                 # 15 Product_EURO_SellingPrice
    sold = []                                 # 16 Product_QuantitySold
    qLeft =[]                                 # 17 Product_QuantityLeft
    shipFrom = []                             # 18 Product_ShippedFrom
    shipTo = []                               # 19 Product_ShippedTo
    image = []                                # 20 Product_Image
    image_vendor = []                         # 21 Vendor_Image
    href = []                                 # 22 Product_Links

    main = soup.find('main', {'id': 'main'})
    products_list = main.find('ul', recursive=False).find_all('li', recursive=False)
    nm = len(products_list)

    for product in products_list:
        # Finding the name of the product
        name_of_product = product.find("h2", {"class": "woocommerce-loop-product__title"}).find("a").text
        name_of_product_cleaned = cleanString(name_of_product.strip())
        # print(name_of_product_cleaned)
        name.append(name_of_product_cleaned)
        #finding the URL
        try:
            url = product.find("a", class_="woocommerce-loop-product__link").get('href')
            href.append(url)
        except AttributeError as e:
            print("I can't find the link")
            raise e

        # Finding Product Image
        product_image = product.find('a', {'class': 'woocommerce-loop-image-link woocommerce-LoopProduct-link woocommerce-loop-product__link'}).find('img')
        product_image = product_image.get('src')
        product_image = product_image.split('base64,')[-1]
        image.append(product_image)

        # Finding BTC Price
        prices = product.find('span', {"class": "price"}).findAll('span', {"class": "cs"})
        price = prices[0].text
        BTC.append(cleanNumbers(price.strip()))

        #everything else appends a -1
        rating_vendor.append("-1")
        USD.append("-1")
        vendor.append('-1')
        success.append("-1")
        CVE.append("-1")
        MS.append("-1")
        category.append("-1")
        describe.append("-1")
        views.append("-1")
        reviews.append("-1")
        addDate.append("-1")
        EURO.append("-1")
        sold.append("-1")
        qLeft.append("-1")
        shipFrom.append("-1")
        shipTo.append("-1")
        image_vendor.append("-1")

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(
        marketplace = mktName,
        nm = nm,
        vendor = vendor,
        rating_vendor = rating_vendor,
        success_vendor = success,
        nombre = name,
        CVE = CVE,
        MS = MS,
        category = category,
        describe = describe,
        views = views,
        reviews = reviews,
        rating_item = rating_item,
        addDate = addDate,
        BTC = BTC,
        USD = USD,
        EURO = EURO,
        sold = sold,
        qLeft = qLeft,
        shipFrom = shipFrom,
        shipTo = shipTo,
        href = href,
        image = image,
        image_vendor = image_vendor
    )


#called by the crawler to get description links on a listing page
#@param: beautifulsoup object that is using the correct html page (listing page)
#return: list of description links from a listing page
def nexus_links_parser(soup):
    # Returning all links that should be visited by the Crawler

    href = []
    # Using a shorter, but still unique, class name
    listing = soup.find_all("a", class_="woocommerce-loop-product__link")

    for a in listing:
        link = a.get('href')
        if link:  # Checks if 'href' attribute is not None
            href.append(link)

    return href
